/**
 * Define api of apeman-api-path
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const path = require('path')
const defaults = require('defaults')
const describe = require('./describe')

let cwd = () => process.cwd()

/** @lends define */
function define (options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let { root } = defaults(options, {
    root: cwd()
  })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    dirname (filename) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        let dirname = path.dirname(...args)
        return yield s.resolve(dirname)
      })
    },

    basename (filename, extname) {
      const s = this
      let args = [ ...arguments ]
      return co(function * () {
        return path.basename(...args)
      })
    },

    relative (from, to) {
      const s = this
      return co(function * () {
        from = yield s.resolve(from)
        to = yield s.resolve(to)
        return path.relative(from, to)
      })
    },

    resolve (...filenames) {
      const s = this
      return co(function * () {
        let resolved = path.resolve(root, ...filenames)
        let isTraversal = resolved.indexOf(root) !== 0
        if (isTraversal) {
          throw new Error('DIRECTORY_TRAVERSAL_DETECTED')
        }
        return path.relative(cwd(), resolved)
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = define
